#include <gtk/gtk.h>
#include <math.h>

#include "gvc-dock-window.h"

struct _GvcDockWindow
{
    GtkWindow parent;
};

G_DEFINE_TYPE(GvcDockWindow, gvc_dock_window, GTK_TYPE_WINDOW)

static void
gvc_dock_window_init (GvcDockWindow *self)
{
}

static gboolean
gvc_dock_window_draw(GtkWidget *widget,
                     cairo_t *cr)
{
    GtkStyleContext *context;
    GtkAllocation alloc;
    GtkStateFlags state;
    gboolean ret = FALSE;
    gdouble degrees = M_PI / 180.0;
    GdkRGBA bg_color;
    gint radius = 6;

    context = gtk_widget_get_style_context (GTK_WIDGET (widget));
    gtk_style_context_save (context);

    gtk_widget_get_allocation(widget, &alloc);
    state = gtk_widget_get_state_flags (widget);

    gtk_style_context_add_class (context, "background");
    gtk_style_context_get_background_color (context,
                                            state,
                                            &bg_color);

    cairo_new_sub_path(cr);
    cairo_arc(cr, alloc.x + alloc.width - radius, alloc.y + radius, radius, -90 * degrees, 0 * degrees);
    cairo_arc(cr, alloc.x + alloc.width - radius, alloc.y + alloc.height - radius, radius, 0 * degrees, 90 * degrees);
    cairo_arc(cr, alloc.x + radius, alloc.y + alloc.height - radius, radius, 90 * degrees, 180 * degrees);
    cairo_arc(cr, alloc.x + radius, alloc.y + radius, radius, 180 * degrees, 270 * degrees);
    cairo_close_path(cr);

    cairo_set_source_rgba (cr, bg_color.red, bg_color.green, bg_color.blue, bg_color.alpha);
    cairo_fill_preserve (cr);
    cairo_clip(cr);

    gtk_style_context_restore (context);

    if (GTK_WIDGET_CLASS(gvc_dock_window_parent_class)->draw)
        ret = GTK_WIDGET_CLASS(gvc_dock_window_parent_class)->draw(widget, cr);

    return ret;
}

static void
gvc_dock_window_class_init (GvcDockWindowClass *class)
{
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(class);

    widget_class->draw = gvc_dock_window_draw;
}

GtkWidget *
gvc_dock_window_new (GtkWindowType type)
{
    return g_object_new (GVC_TYPE_DOCK_WINDOW, "type", type, NULL);
}
